@@ -42,12 +42,26 @@ extern int preserve_uid;
 extern int preserve_gid;
 extern int preserve_times;
 extern int relative_paths;
+extern int copy_links;
 
 static char **local_exclude_list = NULL;
 
 static void clean_fname(char *name);
 
 
+int link_stat(const char *Path, struct stat *Buffer) 
+{
+#if SUPPORT_LINKS
+    if (copy_links) {
+	return stat(Path, Buffer);
+    } else {
+	return lstat(Path, Buffer);
+    }
+#else
+    return stat(Path, Buffer);
+#endif
+}
+
 /*
   This function is used to check if a file should be included/excluded
   from the list of files based on its name and type etc
@@ -68,7 +82,7 @@ static dev_t filesystem_dev;
 static void set_filesystem(char *fname)
 {
   struct stat st;
-  if (lstat(fname,&st) != 0) return;
+  if (link_stat(fname,&st) != 0) return;
   filesystem_dev = st.st_dev;
 }
 
@@ -187,7 +201,7 @@ void receive_file_entry_v11(struct file_struct *file,
   bzero((char *)file,sizeof(*file));
 
   file->name = (char *)malloc(l1+l2+1);
-  if (!file->name) out_of_memory("receive_file_entry");
+  if (!file->name) out_of_memory("receive_file_entry 1");
 
   strncpy(file->name,lastname,l1);
   read_buf(f,file->name+l1,l2);
@@ -203,15 +217,13 @@ void receive_file_entry_v11(struct file_struct *file,
   if (preserve_devices && IS_DEVICE(file->mode))
     file->rdev = (flags & SAME_RDEV) ? last_rdev : (dev_t)read_int(f);
 
-#if SUPPORT_LINKS
   if (preserve_links && S_ISLNK(file->mode)) {
     int l = read_int(f);
     file->link = (char *)malloc(l+1);
-    if (!file->link) out_of_memory("receive_file_entry");
+    if (!file->link) out_of_memory("receive_file_entry 2");
     read_buf(f,file->link,l);
     file->link[l] = 0;
   }
-#endif
 
 #if SUPPORT_HARD_LINKS
   if (preserve_hard_links && S_ISREG(file->mode)) {
@@ -243,7 +255,7 @@ static struct file_struct *make_file(char *fname)
 
   bzero(sum,SUM_LENGTH);
 
-  if (lstat(fname,&st) != 0) {
+  if (link_stat(fname,&st) != 0) {
     fprintf(FERROR,"%s: %s\n",
 	    fname,strerror(errno));
     return NULL;
@@ -418,7 +430,7 @@ struct file_list *send_file_list(int f,int argc,char *argv[])
       strcat(fname,".");
     }
 
-    if (lstat(fname,&st) != 0) {
+    if (link_stat(fname,&st) != 0) {
       fprintf(FERROR,"%s : %s\n",fname,strerror(errno));
       continue;
     }
