@@ -116,8 +116,8 @@ void do_hard_links(struct file_list *flist)
 	hlink_list[i].inode == hlink_list[i-1].inode) {
       struct stat st1,st2;
 
-      if (lstat(hlink_list[i-1].name,&st1) != 0) continue;
-      if (lstat(hlink_list[i].name,&st2) != 0) {
+      if (link_stat(hlink_list[i-1].name,&st1) != 0) continue;
+      if (link_stat(hlink_list[i].name,&st2) != 0) {
 	if (!dry_run && link(hlink_list[i-1].name,hlink_list[i].name) != 0) {
 		if (verbose > 0)
 			fprintf(FINFO,"link %s => %s : %s\n",
