@@ -30,6 +30,7 @@ extern int remote_version;
 
 extern char *backup_suffix;
 
+extern int whole_file;
 extern int block_size;
 extern int update_only;
 extern int make_backups;
@@ -200,7 +201,7 @@ static int set_perms(char *fname,struct file_struct *file,struct stat *st,
   if (dry_run) return 0;
 
   if (!st) {
-    if (lstat(fname,&st2) != 0) {
+    if (link_stat(fname,&st2) != 0) {
       fprintf(FERROR,"stat %s : %s\n",fname,strerror(errno));
       return 0;
     }
@@ -264,7 +265,7 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
   if (verbose > 2)
     fprintf(FERROR,"recv_generator(%s,%d)\n",fname,i);
 
-  statret = lstat(fname,&st);
+  statret = link_stat(fname,&st);
 
   if (S_ISDIR(file->mode)) {
     if (dry_run) return;
@@ -288,8 +289,8 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
     return;
   }
 
-#if SUPPORT_LINKS
   if (preserve_links && S_ISLNK(file->mode)) {
+#if SUPPORT_LINKS
     char lnk[MAXPATHLEN];
     int l;
     if (statret == 0) {
@@ -312,9 +313,9 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
 	fprintf(FINFO,"%s -> %s\n",
 		fname,file->link);
     }
+#endif
     return;
   }
-#endif
 
 #ifdef HAVE_MKNOD
   if (am_root && preserve_devices && IS_DEVICE(file->mode)) {
@@ -404,6 +405,12 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
     return;
   }
 
+  if (whole_file) {
+    write_int(f_out,i);
+    send_sums(NULL,f_out);    
+    return;
+  }
+
   /* open the file */  
   fd = open(fname,O_RDONLY);
 
@@ -533,7 +540,6 @@ static void delete_one(struct file_struct *f)
 static void delete_files(struct file_list *flist)
 {
   struct file_list *local_file_list;
-  char *dot=".";
   int i, j;
   char *last_name=NULL;
 
@@ -566,9 +572,17 @@ static char *cleanup_fname = NULL;
 
 void exit_cleanup(int code)
 {
-  if (cleanup_fname)
-    unlink(cleanup_fname);
-  exit(code);
+	if (cleanup_fname)
+		unlink(cleanup_fname);
+	signal(SIGUSR1, SIG_IGN);
+	if (code) {
+#ifdef GETPGRP_VOID
+		kill(-getpgrp(), SIGUSR1);
+#else
+		kill(-getpgrp(getpid()), SIGUSR1);
+#endif
+	}
+	exit(code);
 }
 
 void sig_int(void)
@@ -656,6 +670,7 @@ int recv_files(int f_in,struct file_list *flist,char *local_name,int f_gen)
       /* open tmp file */
       if (strlen(fname) > (MAXPATHLEN-8)) {
 	fprintf(FERROR,"filename too long\n");
+	close(fd1);
 	continue;
       }
       sprintf(fnametmp,"%s.XXXXXX",fname);
@@ -667,7 +682,7 @@ int recv_files(int f_in,struct file_list *flist,char *local_name,int f_gen)
 	continue;
       }
       fd2 = open(fnametmp,O_WRONLY|O_CREAT,file->mode);
-      if (relative_paths && errno == ENOENT && 
+      if (fd2 == -1 && relative_paths && errno == ENOENT && 
 	  create_directory_path(fnametmp) == 0) {
 	      fd2 = open(fnametmp,O_WRONLY|O_CREAT,file->mode);
       }
@@ -817,6 +832,7 @@ off_t send_files(struct file_list *flist,int f_out,int f_in)
       /* map the local file */
       if (fstat(fd,&st) != 0) {
 	fprintf(FERROR,"fstat failed : %s\n",strerror(errno));
+	close(fd);
 	return -1;
       }
       
